@@ -1,14 +1,15 @@
 package HTML::Template::Compiled;
-# $Id: Compiled.pm 1078 2008-09-09 20:44:07Z tinita $
+# $Id: Compiled.pm 1113 2011-04-27 18:49:59Z tinita $
 # doesn't work with make tardist
 #our $VERSION = ($version_pod =~ m/^\$VERSION = "(\d+(?:\.\d+)+)"/m) ? $1 : "0.01";
-our $VERSION = "0.93";
+our $VERSION = "0.95";
 use Data::Dumper;
 BEGIN {
 use constant D => $ENV{HTC_DEBUG} || 0;
 }
 use strict;
 use warnings;
+use Digest::MD5 qw/ md5_hex /;
 
 our $Storable = 1;
 use Carp;
@@ -20,7 +21,6 @@ use HTML::Template::Compiled::Expression qw(:expressions);
 use HTML::Template::Compiled::Compiler;
 # TODO
 eval {
-    require HTML::Entities;
     require URI::Escape;
 };
 eval {
@@ -52,7 +52,7 @@ use constant PARAM => 0;
 BEGIN {
     my @map = (
         undef, qw(
-          path filename file scalar filehandle
+          path md5_path filename file scalar filehandle
           file_cache cache_dir cache search_path
           loop_context case_sensitive global_vars
           default_path
@@ -160,7 +160,9 @@ sub new_from_perl {
     $self->set_cache( exists $args{cache} ? $args{cache} : 1 );
     $self->set_filename( $args{filename} );
     $self->set_cache_dir( $args{cache_dir} );
+    my $md5path = md5_hex(@{ $args{path} || [] });
     $self->set_path( $args{path} );
+    $self->set_md5_path( $md5path );
     $self->set_scalar( $args{scalarref} );
 
     unless ( $self->get_scalar ) {
@@ -187,7 +189,10 @@ sub new_file {
     $self->set_cache( exists $args{cache} ? $args{cache} : 1 );
     $self->init_cache(\%args);
     #$self->set_cache_dir( $args{cache_dir} );
+    my $md5path = md5_hex(@{ $args{path} || [] });
+    my $test = $args{path};
     $self->set_path( $args{path} );
+    $self->set_md5_path( $md5path );
     if (my $t = $self->from_cache(\%args)) {
         $t->init_includes();
         return $t;
@@ -212,7 +217,9 @@ sub new_filehandle {
     $self->set_cache(0);
     $self->init_cache(\%args);
     #$self->set_cache_dir( $args{cache_dir} );
+    my $md5path = md5_hex(@{ $args{path} || [] });
     $self->set_path( $args{path} );
+    $self->set_md5_path( $md5path );
     if (my $t = $self->from_cache(\%args)) {
         return $t;
     }
@@ -249,9 +256,14 @@ sub new_scalar_ref {
     $self->set_scalar( $args{scalarref} );
     my $text = $self->get_scalar;
     my $md5  = md5($$text);
+    if ($args{cache} and !$md5) {
+        croak "For caching scalarrefs you need Digest::MD5";
+    }
     $self->set_filename($md5);
     D && $self->log("md5: $md5");
+    my $md5path = md5_hex(@{ $args{path} || [] });
     $self->set_path( $args{path} );
+    $self->set_md5_path( $md5path );
     if (my $t = $self->from_cache(\%args)) {
         return $t;
     }
@@ -268,6 +280,7 @@ sub init_includes {
     for my $fullpath (keys %$includes) {
         my ($path, $filename, $htc) = @{ $includes->{$fullpath} };
         D && $self->log("checking $fullpath ($filename) $htc?");
+        $cache .= '-' . $self->get_md5_path;
         if (HTML::Template::Compiled::needs_new_check(
                 $cache||'',$filename)
         ) {
@@ -333,6 +346,7 @@ sub from_cache {
     if ( $self->get_cache ) {
         my $dir = $self->get_cache_dir;
         $dir = '' unless defined $dir;
+        $dir .= '-' . $self->get_md5_path;
         my $fname  = $self->get_filename;
         $t = $self->from_mem_cache($dir,$fname);
         if ($t) {
@@ -400,6 +414,8 @@ sub from_cache {
         D && $self->stack(1);
         my $dir = $self->get_cache_dir;
         $dir = '' unless defined $dir;
+        my @c = caller();
+        $dir .= '-' . $self->get_md5_path;
         my $fname = $self->get_filename;
         D && $self->log( "add_mem_cache $fname" );
         my $clone = $self->clone;
@@ -553,6 +569,16 @@ sub add_file_cache {
                 checked => $times{checked},
             },
         };
+        # TODO Storable cannot store qr//
+        # but as we don't need those serialized anyway
+        # we let Storable just store a dummy entry
+        local $Storable::forgive_me = 1;
+        local $SIG{__WARN__} = sub {
+            my ($w) = @_;
+            unless ($w =~ m/Can't store item REGEXP/) {
+                warn $w;
+            }
+        };
         Storable::store($to_cache, "$cachefile.storable");
     }
     else {
@@ -1142,7 +1168,9 @@ sub new_from_object {
     $new->set_includes({});
     $new->set_scalar();
     $new->set_filehandle();
+    my $md5path = md5_hex(@{ $path || [] });
     $new->set_path($path);
+    $new->set_md5_path( $md5path );
     $new->set_perl(undef);
     if (my $cached = $new->from_cache) {
         $cached->set_plugins($self->get_plugins);
@@ -1536,7 +1564,7 @@ HTML::Template::Compiled - Template System Compiles HTML::Template files to Perl
 
 =head1 VERSION
 
-$VERSION = "0.93"
+$VERSION = "0.95"
 
 =cut
 
@@ -2211,6 +2239,14 @@ outputs
 
 So specifying BREAK=3 sets __break__ to 1 every 3rd loop iteration.
 
+TMPL_LOOP expects an array reference, also if it is a method call. If
+you want to iterate with TMPL_LOOP over a list from a method call, set
+the attribute C<context=list>:
+
+    <tmpl_loop object.list_method context=list>
+        <tmpl_var _ >
+    </tmpl_loop>
+
 =head2 TMPL_WHILE
 
 Useful for iterating, for example over database resultsets.
@@ -2255,6 +2291,14 @@ can also sort the output:
 You have to set the option C<loop_context_vars> to true to use
 the special vars C<__key__> and C<__value__>.
 
+If you want to iterate over a hash instead of a hashref (some
+methods might return plain hashes instead of references and
+TMPL_EACH expects a ref), then you can set C<context=list>:
+
+    <tmpl_each object.hash_method context=list>
+    <tmpl_var __key__ >
+    </tmpl_each>
+
 =head2 TMPL_COMMENT
 
 For debugging purposes you can temporarily comment out regions:
@@ -2596,7 +2640,7 @@ you want to use {C<{if foo}}{{var bar}}{{/if foo}}>, then your
 definition should be:
 
     [
-        qr({{), start of opening tag
+        qr({{), # start of opening tag
         qr(}}), # end of opening tag
         qr({{/), # start of closing tag
         qr(}}), # end of closing tag
@@ -2957,7 +3001,7 @@ Special Thanks to Sascha Kiefer - he finds all the bugs!
 Ronnie Neumann, Martin Fabiani, Kai Sengpiel, Jan Willamowius, Justin Day,
 Steffen Winkler, Henrik Tougaard for ideas, beta-testing and patches
 
-perlmonks.org and perl-community.de for everyday learning
+L<http://www.perlmonks.org/> and http://www.perl-community.de/> for everyday learning
 
 Corion, Limbic~Region, tye, runrig and others from perlmonks.org 
 
